import { getApolloServer } from '@nestjs/apollo';
import { INestApplication } from '@nestjs/common';
import { Test } from '@nestjs/testing';
import { AppModule } from "../app.module";
import { ApolloServer, BaseContext } from "@apollo/server";
import { DocumentNode } from 'graphql';
import { gql } from 'apollo-server-core';

export class ApolloTestServer {

    private app: INestApplication;
    private apolloServer: ApolloServer<BaseContext>;

    constructor() { }

    public async init() {
        const testingModule = Test.createTestingModule({imports: [AppModule]});
        const fixture = await testingModule.compile();
        this.app = fixture.createNestApplication();
        await this.app.init();
        this.apolloServer = getApolloServer(this.app);
    }

    public async close() {
        await this.app.close();
    }

    public query(queryString: string) {
        const query = "query Query {" + queryString + "}";
        return gql`${query}`;
    }

    public mutation(queryString: string) {
        const query = "mutation Mutation {" + queryString + "}";
        return gql`${query}`;
    }

    public async executeQuery(query: DocumentNode | string) {
        if (typeof query === 'string') query = this.query(query);
        const result: any = await this.apolloServer.executeOperation({query});
        return {
            data: result?.body?.singleResult?.data || null,
            errors: result?.body?.singleResult?.errors || null,
        }
    }

    public async executeMutation(query: DocumentNode | string) {
        if (typeof query === 'string') query = this.mutation(query);
        const result: any = await this.apolloServer.executeOperation({query});
        return {
            data: result?.body?.singleResult?.data || null,
            errors: result?.body?.singleResult?.errors || null,
        }
    }

}